#!/bin/sh

set -e

if [ -n "$CCLUB_CONFIG_DASH_X" ]; then
    set -x
fi
force=
if [ -n "$CCLUB_CONFIG_FORCE" ]; then
    force=1
fi

CCLUB_KEYRING_URL=http://debian.club.cc.cmu.edu/debian/dists/bookworm-cclub/contrib/binary-amd64/cclub-keyring_0.11_all.deb
CCLUB_KEYRING_SHA256SUM=54790d57756b4ede1f78dbff59c30a08a8653edafbf5e043ec8396b50fa8e4b8

ELTS_KEYRING_URL=http://deb.freexian.com/extended-lts/pool/main/f/freexian-archive-keyring/freexian-archive-keyring_2022.06.08_all.deb
ELTS_KEYRING_SHA256SUM=a8160d1aa1a40aa9988bf0b389b650550c7460ec3b4ec1d847778fe44b9c4dbc


# Trap handler to remove temporary files and directories.

install=1
download_dir=
download_file=

cleanup() {
    if [ -n "$install" ] && [ -n "$download_dir" ]; then
        rm -rf "$download_dir"
    fi
    if [ -n "$download_file" ]; then
        rm -f "$download_file"
    fi
}


# Argument processing

force=
install=1
want=
while [ $# -gt 0 ]; do
    case "$1" in
    --download-only)
        install=
        ;;

    --force)
        force=1
        ;;

    --want)
        want="$want $2"
        shift
        ;;

    --*|-*)
        echo "Invalid argument \"$1\"" >&2
        exit 1
        ;;

    *)
        break
        ;;

    esac

    shift
done

if [ $# -eq 0 ]; then
    :
elif [ $# -eq 1 ] && [ x"$1" = x"--" ]; then
    :
else
    echo "Unexpected argument(s) \"$*\"" >&2
    exit 1
fi

# Check for artifacts associated with the keyrings we can fetch.

have_cclub=
cclub_glob=`printf '%s\n' /etc/apt/trusted.gpg.d/cclub-*.gpg`
if [ x"$cclub_glob" = x"${cclub_glob%\**}" ]; then
    have_cclub=1
fi

have_elts=
if [ -e  /etc/apt/trusted.gpg.d/freexian-archive-extended-lts.gpg ]; then
    have_elts=1
fi

# Figure out what we need to get

set -- $want
need=
while [ $# -gt 0 ]; do
    if [ -n "$force" ] || eval "[ -z \"\$have_$1\" ]"; then
        need="$need $1"
        eval "need_$1=1"
    else
        eval "need_$1="
    fi
    shift
done


# If we don't need anything, we're done.

if [ -z "$need" ]; then
    exit 0;
fi

# Otherwise, get everything we need.

download_dir=.
if [ -n "$install" ]; then
    download_dir=`mktemp --tmpdir -d get_keyrings_XXXXXX`
fi

trap cleanup EXIT

new_perms=
download() {
    local url="$1"
    local sha256sum="$2"
    local file="${url##*/}"
    local target="${download_dir}/${file}"
    local umask

    if [ -z "$install" ]; then
        download_file=`mktemp "${download_dir}/.${file}.XXXXXX"`
        target="$download_file"
        if [ -z "$new_perms" ]; then
            umask=`umask`
            case "$umask" in
            *[01]?) new_perms=6 ;;
            *[23]?) new_perms=4 ;;
            *[45]?) new_perms=2 ;;
            *[67]?) new_perms=0 ;;
            esac
            case "$umask" in
            *[01]) new_perms="${new_perms}6" ;;
            *[23]) new_perms="${new_perms}4" ;;
            *[45]) new_perms="${new_perms}2" ;;
            *[67]) new_perms="${new_perms}0" ;;
            esac
        fi
    fi

    wget -nv -O "$target" "$url"
    if [ x"`sha256sum < "$target"`" != x"$sha256sum  -" ]; then
        echo "Hash mismatch for ${file}, aborting!" >&2
        exit 1
    fi

    if [ -z "$install" ]; then
        chmod "+0${new_perms}" "$download_file"
        mv -fT "$download_file" "${download_dir}/${file}"
        download_file=
    fi
}

if [ -z "$http_proxy" ] && [ -z "$HTTP_PROXY" ]; then
    http_proxy=http://pkg-proxy.club.cc.cmu.edu:8000/
    no_proxy=.club.cc.cmu.edu,.club.cc.local.cmu.edu
    export http_proxy no_proxy
fi

if [ -n "$need_cclub" ]; then
    download "$CCLUB_KEYRING_URL" "$CCLUB_KEYRING_SHA256SUM"
    if [ -n "$install" ]; then
        dpkg -i "$download_dir/${CCLUB_KEYRING_URL##*/}"
    fi
fi
if [ -n "$need_elts" ]; then
    download "$ELTS_KEYRING_URL" "$ELTS_KEYRING_SHA256SUM"
    if [ -n "$install" ]; then
        dpkg -i "$download_dir/${ELTS_KEYRING_URL##*/}"
    fi
fi

exit 0
